@@ -111,6 +111,48 @@ class LogicalOp(object):
         return '%s(%r, parent=%r)' % (self.flag, self.right, self.parent)
 
 
+def reduce_cmp(right):
+    if isinstance(right, _ast.Return):
+        return right.value
+    if isinstance(right, LogicalOp) and right.flag == 'AND':
+        right.right = reduce_cmp(right.right)
+        if isinstance(right.right, _ast.Compare) and isinstance(right.parent, _ast.Compare):
+            if right.parent.comparators[-1] is  right.right.left:
+                right.parent.ops.extend(right.right.ops)
+                right.parent.comparators.extend(right.right.comparators)
+                right = right.parent
+            
+    return right
+
+
+
+def JUMP_IF_X_OR_POP(operand):
+    def JUMP_IF_Y_OR_POP(self, instr):
+        left = self.pop_ast_item()
+        and_block = self.gather_jumps(instr)
+        hi = self.process_logic([instr] + and_block)
+        
+        # Compress (a > b) and (b > c) into (a > b > c)
+        hi.right = reduce_cmp(hi.right)
+        
+        if isinstance(hi.right, _ast.Compare):
+            if isinstance(left, _ast.Compare) and left.comparators[-1] is hi.right.left:
+                left.ops.extend(hi.right.ops)
+                left.comparators.extend(hi.right.comparators)
+            else:
+                left = _ast.BoolOp(op=operand(), values=[left, hi.right], lineno=instr.lineno, col_offset=0)
+        else:
+            bool_op, insert_into = parse_logic(hi)
+            insert_into.insert(0, left)
+            left = bool_op
+             
+        self.push_ast_item(left)
+        
+        if len(self.ilst) >= 2 and [ii.opname for ii in self.ilst[:2]] == ['ROT_TWO', 'POP_TOP']:
+            self.ilst.pop(0)
+            self.ilst.pop(0)
+    return JUMP_IF_Y_OR_POP
+
 class CtrlFlowInstructions(object):
 
     def split_handlers(self, handlers_blocks):
@@ -169,7 +211,7 @@ class CtrlFlowInstructions(object):
                 exc_body = self.decompile_block(handler_body[:-1]).stmnt()
                 if not exc_body:
                     exc_body.append(_ast.Pass(lineno=except_instrs[-2].lineno, col_offset=0))
-                #is this for python 3?
+                # is this for python 3?
                 if py3 and exc_name is not None:
                     exc_name = exc_name.id
                     
@@ -222,7 +264,7 @@ class CtrlFlowInstructions(object):
 
         try_finally = _ast.TryFinally(body=try_except, finalbody=finally_)
         
-        self.ast_stack.append(try_finally)
+        self.push_ast_item(try_finally)
 
 #    @py3op
     def do_except_block(self, block):
@@ -240,7 +282,7 @@ class CtrlFlowInstructions(object):
 
         body = self.decompile_block(handler_block).stmnt()
         
-        self.ast_stack.extend(body)
+        self._ast_stack.extend(body)
         
 #    @py2op
 #    def SETUP_FINALLY(self, instr):
@@ -273,8 +315,8 @@ class CtrlFlowInstructions(object):
 #        print("finally_", finally_)
 #        print_ast(finally_[0])
 #        print()
-##        print_ast(finally_[1])
-##        print("\n\n")
+# #        print_ast(finally_[1])
+# #        print("\n\n")
 #        
 #        try_except = self.decompile_block(try_except_block).stmnt()
         
@@ -295,7 +337,7 @@ class CtrlFlowInstructions(object):
 
         end, handlers = self.split_handlers(handlers_blocks)
         
-        #raise exception in python 3 (python 2 ilst does not include end so else may go beond)
+        # raise exception in python 3 (python 2 ilst does not include end so else may go beond)
         else_block = self.make_block(end, inclusive=False, raise_=py3)
 
         else_stmnts = self.decompile_block(else_block).stmnt()
@@ -306,7 +348,7 @@ class CtrlFlowInstructions(object):
 
         try_except = _ast.TryExcept(body=body, handlers=handlers, orelse=else_, lineno=instr.lineno, col_offset=0)
 
-        self.ast_stack.append(try_except)
+        self.push_ast_item(try_except)
 
     @SETUP_EXCEPT.py3op
     def SETUP_EXCEPT(self, instr):
@@ -336,7 +378,7 @@ class CtrlFlowInstructions(object):
 
         try_except = _ast.TryExcept(body=body, handlers=handlers, orelse=else_, lineno=instr.lineno, col_offset=0)
 
-        self.ast_stack.append(try_except)
+        self.push_ast_item(try_except)
     
     @py3op
     def POP_EXCEPT(self, instr):
@@ -352,7 +394,7 @@ class CtrlFlowInstructions(object):
             self.while_loop(instr, loop_block)
 
     def BREAK_LOOP(self, instr):
-        self.ast_stack.append(_ast.Break(lineno=instr.lineno, col_offset=0))
+        self.push_ast_item(_ast.Break(lineno=instr.lineno, col_offset=0))
 
     def for_loop(self, loop_block):
         iter_block, _, body_else_block = split(loop_block, 'GET_ITER')
@@ -391,7 +433,7 @@ class CtrlFlowInstructions(object):
         assign = assign_.targets[0]
         for_ = _ast.For(target=assign, iter=iter_stmnt, body=body, orelse=else_, lineno=iter_stmnt.lineno, col_offset=0)
 
-        self.ast_stack.append(for_)
+        self.push_ast_item(for_)
 
     def make_list_comp(self, get_iter, for_iter):
 
@@ -412,10 +454,11 @@ class CtrlFlowInstructions(object):
     
             assert isinstance(assign, _ast.Assign)
     
-            list_expr = self.ast_stack.pop()
+    
+            list_expr = self.pop_ast_item()
     
             # empty ast.List object
-            list_ = self.ast_stack.pop()
+            list_ = self.pop_ast_item()
     
             ifs = []
             elt = refactor_ifs(stmnts[0], ifs)
@@ -430,20 +473,20 @@ class CtrlFlowInstructions(object):
             
             list_comp = _ast.ListComp(elt=elt, generators=generators, lineno=get_iter.lineno, col_offset=0)
         else:
-            list_expr = self.ast_stack.pop()
+            list_expr = self.pop_ast_item()
             list_comp = stmnts[0]
             
             generators = list_comp.generators
             
             # empty ast.List object
-            list_ = self.ast_stack.pop()
+            list_ = self.pop_ast_item()
             if not isinstance(list_, _ast.Assign):
                 comp = _ast.comprehension(target=list_.targets[0], iter=None, ifs=[], lineno=get_iter.lineno, col_offset=0)
                 generators.insert(0, comp)
                 
             generators[0].iter = list_expr
 
-        self.ast_stack.append(list_comp)
+        self.push_ast_item(list_comp)
 
     def extract_listcomp(self, function, sequence):
 
@@ -460,7 +503,7 @@ class CtrlFlowInstructions(object):
                 generator.iter = sequence
         
         setcomp = _ast.ListComp(elt=value.elt, generators=generators, lineno=value.lineno, col_offset=0)
-        self.ast_stack.append(setcomp)
+        self.push_ast_item(setcomp)
         
     def extract_setcomp(self, function, sequence):
         
@@ -477,7 +520,7 @@ class CtrlFlowInstructions(object):
                 generator.iter = sequence
         
         setcomp = _ast.SetComp(elt=value.elt, generators=generators, lineno=value.lineno, col_offset=0)
-        self.ast_stack.append(setcomp)
+        self.push_ast_item(setcomp)
 
     def extract_dictcomp(self, function, sequence):
 
@@ -494,7 +537,7 @@ class CtrlFlowInstructions(object):
                 generator.iter = sequence
         
         setcomp = _ast.DictComp(key=value.elt[0], value=value.elt[1], generators=generators, lineno=value.lineno, col_offset=0)
-        self.ast_stack.append(setcomp)
+        self.push_ast_item(setcomp)
 #
 #        assert len(function.code.nodes) == 1
 #        assert isinstance(function.code.nodes[0], _ast.Return)
@@ -510,7 +553,7 @@ class CtrlFlowInstructions(object):
 #            qual.list = sequence
 #
 #        setcomp = _ast.DictComp(key=key, value=value, generators=quals, lineno=value.lineno, col_offset=0)
-#        self.ast_stack.append(setcomp)
+#        self.push_ast_item(setcomp)
 
     def GET_ITER(self, instr):
         for_iter = self.ilst.pop(0)
@@ -519,8 +562,8 @@ class CtrlFlowInstructions(object):
             call_function = for_iter
             assert call_function.oparg == 1
 
-            sequence = self.ast_stack.pop()
-            function = self.ast_stack.pop()
+            sequence = self.pop_ast_item()
+            function = self.pop_ast_item()
 
             if function.name == '<listcomp>':
                 self.extract_listcomp(function, sequence)
@@ -542,17 +585,17 @@ class CtrlFlowInstructions(object):
         pass
 
     def MAP_ADD(self, instr):
-        key = self.ast_stack.pop()
-        value = self.ast_stack.pop()
+        key = self.pop_ast_item()
+        value = self.pop_ast_item()
 
-        self.ast_stack.append((key, value))
+        self.push_ast_item((key, value))
         'NOP'
 
     def SET_ADD(self, instr):
         'NOP'
 
     def FOR_ITER(self, instr):
-        #set or dict comp
+        # set or dict comp
         self.make_list_comp(instr, instr)
 
     def while_loop(self, instr, loop_block):
@@ -608,7 +651,7 @@ class CtrlFlowInstructions(object):
 
         while_ = _ast.While(test=test, body=body_, orelse=else_, **kw)
 
-        self.ast_stack.append(while_)
+        self.push_ast_item(while_)
 
 
     def gather_jumps(self, jump_instr):
@@ -637,7 +680,7 @@ class CtrlFlowInstructions(object):
                 new_block = self.make_block(to=old_max, inclusive=False, raise_=False)
                 and_block.extend(new_block)
 
-        #print("and_block", and_block)
+        # print("and_block", and_block)
         return and_block
 
     def process_logic(self, logic_block):
@@ -653,39 +696,58 @@ class CtrlFlowInstructions(object):
                 else:
                     right = self.process_logic(logic_block[1:])
                 parent = None
-#                assert False
             else:
                 right = self.process_logic(logic_block[1:idx - 1])
                 parent = self.process_logic(logic_block[idx - 1:])
 
-#            if right is None:
             return LogicalOp(flag, right, parent, jump_instr.lineno)
         else:
             idx = find_index(logic_block, lambda instr: instr.opname in JUMPS, default=None)
 
             if idx is None:
+                if len(logic_block) == 3 and [ii.opname for ii in logic_block[1:]] == ['ROT_TWO', 'POP_TOP']:
+                    assert logic_block[0].opname in ['JUMP_FORWARD', 'RETURN_VALUE'], logic_block 
+                    return None
                 stmnts = self.decompile_block(logic_block).stmnt()
-#                assert len(stmnts) == 1
-                return stmnts[0]
+                if len(stmnts) > 1:
+                    self._ast_stack.extend(stmnts[:-1]) 
+                return stmnts[-1]
+            
             else:
                 right = logic_block[idx:]
                 parent = logic_block[:idx]
 
-                stmnts = self.decompile_block(parent).stmnt()
-                assert len(stmnts) == 1
-                parent = stmnts[0]
+                if len(parent) == 3 and [ii.opname for ii in parent[1:]] == ['ROT_TWO', 'POP_TOP']:
+                    assert parent[0].opname in ['JUMP_FORWARD', 'RETURN_VALUE'], parent
+                    parent = None
+                else:
+                    stmnts = self.decompile_block(parent).stmnt()
+                    
+                    if len(stmnts) == 1:
+                        parent = stmnts[-1]
+                    else:
+                        parent = stmnts[-1]
+                        self._ast_stack.extend(stmnts[:-1]) 
+                        
+                    parent = stmnts[-1]
 
                 right = self.process_logic(right)
-
-                assert right.parent is None
-
+                
+                if parent is not None:
+                    if right.parent is None:
+                        right.parent = parent
+                    else:
+                        right, to_insrt = parse_logic(right)
+                        to_insrt.insert(0, parent)
+                        return right
+                        
                 if right.right is None:
                     return parent
 
-                right.parent = parent
                 return right
 
 
+
     def logic_ast(self, instr, left, hi):
 #        flag = 'OR' if opname[instr.op] in OR_JUMPS else 'AND'
 
@@ -695,14 +757,6 @@ class CtrlFlowInstructions(object):
 
         return ast_
 
-    def JUMP_IF_TRUE_OR_POP(self, instr):
-        left = self.ast_stack.pop()
-
-        and_block = self.gather_jumps(instr)
-        hi = self.process_logic([instr] + and_block)
-        ast_ = self.logic_ast(instr, left, hi)
-        self.ast_stack.append(ast_)
-
     def make_if(self, instr, left, and_block):
         block = [instr] + and_block[:-1]
 
@@ -734,7 +788,7 @@ class CtrlFlowInstructions(object):
         
         if jump.is_jump:
             else_block = self.make_block(jump.to, inclusive=False, raise_=False)
-        else: # it is a return
+        else:  # it is a return
             else_block = []
 
         if len(else_block):
@@ -751,12 +805,12 @@ class CtrlFlowInstructions(object):
 
         if_ = _ast.If(test=cond, body=body, orelse=else_, lineno=instr.lineno, col_offset=0)
         
-        self.ast_stack.append(if_)
+        self.push_ast_item(if_)
 
 
     def POP_JUMP_IF_TRUE(self, instr):
         
-        left = self.ast_stack.pop()
+        left = self.pop_ast_item()
 
         and_block = self.gather_jumps(instr)
 
@@ -767,46 +821,38 @@ class CtrlFlowInstructions(object):
         else:
             hi = self.process_logic([instr] + and_block)
             ast_ = self.logic_ast(instr, left, hi)
-            self.ast_stack.append(ast_)
+            self.push_ast_item(ast_)
 
     def POP_JUMP_IF_FALSE(self, instr):
         
-        #print("POP_JUMP_IF_FALSE")
-        
-        left = self.ast_stack.pop()
+        left = self.pop_ast_item()
 
         and_block = self.gather_jumps(instr)
-        #This is an IF statement
-        
+        # This is an IF statement
         if and_block[-1].opname in ['JUMP_FORWARD', 'JUMP_ABSOLUTE', 'RETURN_VALUE']:
             
-            #this happens if the function was going to return anyway
+            # this happens if the function was going to return anyway
             if and_block[-1].opname == 'RETURN_VALUE':
                 JUMP_FORWARD = Instruction(and_block[-1].i, 110, lineno=0)
                 JUMP_FORWARD.arg = instr.to
                 and_block.append(JUMP_FORWARD)
             
-            #print()
-            #print("make_if", instr, left, and_block)
-            #print()
+            # print()
+            # print("make_if", instr, left, and_block)
+            # print()
             self.make_if(instr, left, and_block)
             return
-        else: #This is an expression
+        else:  # This is an expression
             hi = self.process_logic([instr] + and_block)
             ast_ = self.logic_ast(instr, left, hi)
-            self.ast_stack.append(ast_)
+            self.push_ast_item(ast_)
 
-    def JUMP_IF_FALSE_OR_POP(self, instr):
-        left = self.ast_stack.pop()
-
-        and_block = self.gather_jumps(instr)
-        hi = self.process_logic([instr] + and_block)
-        ast_ = self.logic_ast(instr, left, hi)
-        self.ast_stack.append(ast_)
+    JUMP_IF_FALSE_OR_POP = JUMP_IF_X_OR_POP(_ast.And)
+    JUMP_IF_TRUE_OR_POP = JUMP_IF_X_OR_POP(_ast.Or)
 
     def JUMP_ABSOLUTE(self, instr):
         continue_ = _ast.Continue(lineno=instr.lineno, col_offset=0)
-        self.ast_stack.append(continue_)
+        self.push_ast_item(continue_)
 
     def JUMP_FORWARD(self, instr):
         pass
@@ -834,14 +880,14 @@ class CtrlFlowInstructions(object):
 
         body = with_
 
-        expr = self.ast_stack.pop()
+        expr = self.pop_ast_item()
 
         with_ = _ast.With(context_expr=expr, optional_vars=as_, body=body,
                           lineno=instr.lineno, col_offset=0)
 
-        self.ast_stack.append(with_)
+        self.push_ast_item(with_)
         
     def CONTINUE_LOOP(self, instr):
         cont = _ast.Continue(lineno=instr.lineno, col_offset=0)
-        self.ast_stack.append(cont)
+        self.push_ast_item(cont)
          
